class SomeExampleOfCollectionModel:
    def __init__(self,
        someExampleOf = None,
        someExampleOfChild = None,
        numberAttribute = None,
        stringAttribute = None
    ):
        self.someExampleOf = someExampleOf
        self.someExampleOfChild = someExampleOfChild
        self.numberAttribute = numberAttribute
        self.stringAttribute = stringAttribute
