from python_framework.api.src.converter.static import StaticConverter

###- deprecated
getValueOrDefault = StaticConverter.getValueOrDefault

###- deprecated
overrideDateData = StaticConverter.overrideDateData

###- deprecated
overrideUserData = StaticConverter.overrideUserData

###- deprecated
overrideData = StaticConverter.overrideData

###- deprecated
to = StaticConverter.to

###- deprecated
toList = StaticConverter.toList

###- deprecated
toResponseDto = StaticConverter.to

###- deprecated
toResponseDtoList = StaticConverter.toList
