from python_helper import Constant as c

REQUEST = 'Request'
RESPONSE = 'Response'

CONTROLLER_SPACE =  f'[CONTROLLER]{c.SPACE}'
CLIENT_SPACE =      f'[CLIENT    ]{c.SPACE}'
LISTENER_SPACE =    f'[LISTENER  ]{c.SPACE}'
EMITTER_SPACE =     f'[EMITTER   ]{c.SPACE}'

CONTROLLER_REQUEST = f'{CONTROLLER_SPACE}{REQUEST}'
CLIENT_REQUEST = f'{CLIENT_SPACE}{REQUEST}'
LISTENER_REQUEST = f'{LISTENER_SPACE}{REQUEST}'
EMITTER_REQUEST = f'{EMITTER_SPACE}{REQUEST}'

CLIENT_RESPONSE = f'{CLIENT_SPACE}{RESPONSE}'
CONTROLLER_RESPONSE = f'{CONTROLLER_SPACE}{RESPONSE}'
LISTENER_RESPONSE = f'{LISTENER_SPACE}{RESPONSE}'
EMITTER_RESPONSE = f'{EMITTER_SPACE}{RESPONSE}'
